@@ -317,7 +317,8 @@ module ActiveLdap
           do_in_timeout(@timeout, &block)
         rescue Timeout::Error => e
           @logger.error {_('Requested action timed out.')}
-          if @retry_on_timeout and retry_limit < 0 and n_retries <= retry_limit
+          if @retry_on_timeout and (retry_limit < 0 or n_retries <= retry_limit)
+            n_retries += 1
             if connecting?
               retry
             elsif try_reconnect
@@ -336,7 +337,7 @@ module ActiveLdap
       def sasl_bind(bind_dn, options={})
         # Get all SASL mechanisms
         mechanisms = operation(options) do
-          root_dse_values("supportedSASLMechanisms")
+          root_dse_values("supportedSASLMechanisms", options)
         end
 
         if options.has_key?(:sasl_quiet)
@@ -579,9 +580,10 @@ module ActiveLdap
           options[:reconnect_attempts] = 0 if force
           options[:reconnect_attempts] += 1 if retry_limit >= 0
           begin
+            options[:try_reconnect] = false
             connect(options)
             break
-          rescue AuthenticationError
+          rescue AuthenticationError, Timeout::Error
             raise
           rescue => detail
             @logger.error do
@@ -633,10 +635,17 @@ module ActiveLdap
 
       def root_dse(attrs, options={})
         found_attributes = nil
+        if options.has_key?(:try_reconnect)
+           try_reconnect = options[:try_reconnect]
+        else
+           try_reconnect = true
+        end
+
         search(:base => "",
                :scope => :base,
                :attributes => attrs,
-               :limit => 1) do |dn, attributes|
+               :limit => 1,
+               :try_reconnect => try_reconnect) do |dn, attributes|
           found_attributes = attributes
         end
         found_attributes
