@@ -679,21 +679,17 @@ module ActiveLdap
         self.classes = initial_classes
         self.dn = attributes
       when Hash
-        classes, attributes = extract_object_class(attributes)
+        attributes = attributes.clone
+        classes = extract_object_class!(attributes)
         self.classes = classes | initial_classes
-        normalized_attributes = {}
-        attributes.each do |key, value|
-          real_key = to_real_attribute_name(key) || key
-          normalized_attributes[real_key] = value
-        end
-        self.dn = normalized_attributes.delete(dn_attribute)
-        self.attributes = normalized_attributes
+        self.attributes = attributes
       else
         format = _("'%s' must be either nil, DN value as ActiveLdap::DN, " \
                    "String or Array or attributes as Hash")
         raise ArgumentError, format % attributes.inspect
       end
       yield self if block_given?
+      run_callbacks :initialize unless _initialize_callbacks.empty?
     end
 
     # Returns true if the +comparison_object+ is the same object, or is of
@@ -831,15 +827,11 @@ module ActiveLdap
       assign_attributes(new_attributes)
     end
 
-    def assign_attributes(new_attributes, options={})
+    def assign_attributes(new_attributes)
       return if new_attributes.blank?
 
       _schema = _local_entry_attribute = nil
-      if options[:without_protection]
-        targets = new_attributes
-      else
-        targets = sanitize_for_mass_assignment(new_attributes, options[:role])
-      end
+      targets = sanitize_for_mass_assignment(new_attributes)
       targets.each do |key, value|
         setter = "#{key}="
         unless respond_to?(setter)
@@ -940,6 +932,17 @@ module ActiveLdap
       end
     end
 
+    def clear_removed_attributes_data(removed_attributes)
+      return if removed_attributes.empty?
+      removed_entry_attribute = EntryAttribute.new(nil, [])
+      removed_attributes.each do |attribute|
+        removed_entry_attribute.register(attribute)
+      end
+      @data.reject! do |key, _|
+        removed_entry_attribute.exist?(key)
+      end
+    end
+
     def schema
       @schema ||= super
     end
@@ -1032,18 +1035,31 @@ module ActiveLdap
       @local_entry_attribute ||= connection.entry_attribute([])
     end
 
-    def extract_object_class(attributes)
+    def extract_object_class!(attributes)
       classes = []
-      attrs = {}
-      attributes.each do |key, value|
-        key = key.to_s
-        if /\Aobject_?class\z/i =~ key
-          classes.concat(value.to_a)
+      attributes.keys.each do |key|
+        string_key = key.to_s
+        if /\Aobject_?class\z/i =~ string_key
+          classes.concat(attributes[key].to_a)
+          attributes.delete(key)
+        end
+      end
+      classes
+    end
+
+    def remove_dn_attribute!(attributes)
+      _dn_attribute = dn_attribute
+      attributes.keys.each do |key|
+        case key
+        when "id", :id, "dn", :dn
+          attributes.delete(key)
         else
-          attrs[key] = value
+          normalized_key = to_real_attribute_name(key) || key
+          if normalized_key == _dn_attribute
+            attributes.delete(key)
+          end
         end
       end
-      [classes, attributes]
     end
 
     def init_base
@@ -1061,9 +1077,11 @@ module ActiveLdap
       @new_entry = false
       @dn_is_base = false
       @ldap_data = attributes
-      classes, attributes = extract_object_class(attributes)
+      attributes = attributes.clone
+      classes = extract_object_class!(attributes)
       self.classes = classes
       self.dn = dn
+      remove_dn_attribute!(attributes)
       initialize_attributes(attributes)
       yield self if block_given?
     end
@@ -1181,8 +1199,9 @@ module ActiveLdap
 
     def split_dn_value(value)
       dn_value = relative_dn_value = nil
+      value = value.first if value.is_a?(Array) and value.size == 1
+      dn_value = value if value.is_a?(DN)
       begin
-        dn_value = value if value.is_a?(DN)
         dn_value ||= DN.parse(value)
       rescue DistinguishedNameInvalid
         begin
